from nose.tools import eq_

from ..propagate_defaults import propagate_defaults


def test_propagate_defaults():

    input = {
        'foos': {
            'defaults': {
                'bar': 1,
                'baz': 2
            },
            '1_foo': {},
            '2_foo': {
                'baz': 3
            }
        }
    }
    expected = {
        'foos': {
            'defaults': {
                'bar': 1,
                'baz': 2
            },
            '1_foo': {
                'bar': 1,
                'baz': 2
            },
            '2_foo': {
                'bar': 1,
                'baz': 3
            }
        }
    }
    eq_(propagate_defaults(input), expected)



from .import_module import import_module
from .import_path import import_path
from .load import load, loads
from .merge import merge
from .propagate_defaults import propagate_defaults
from .about import (__name__, __version__, __author__, __author_email__,
                    __description__, __license__, __url__)


__all__ = [import_module, import_path, load, loads, merge, propagate_defaults,
           __name__, __version__, __author__, __author_email__,
           __description__, __license__, __url__]



from nose.tools import eq_

from ..import_path import import_path
from .util import submodule, value


def test_import_path():
    my_submodule = import_path("yamlconf.tests.util.submodule")
    eq_(submodule, my_submodule)

    my_attribute = import_path("yamlconf.tests.util.submodule.attribute")
    eq_(submodule.attribute, my_attribute)

    my_sub_attribute = import_path(
        "yamlconf.tests.util.submodule.attribute.attribute")
    eq_(submodule.attribute.attribute, my_sub_attribute)

    my_super_sub_attribute = import_path(
        "yamlconf.tests.util.submodule.attribute.attribute.attribute")
    eq_(submodule.attribute.attribute.attribute, my_super_sub_attribute)
    eq_(my_super_sub_attribute.attribute, "hello")

    my_value = import_path("yamlconf.tests.util.value")
    eq_(value, my_value)
    eq_(my_value, "foo")



import os

from nose.tools import eq_, raises

from ..load import load, loads
from .. import errors

EXPECTED = {
    'hats': {
        'red': {
            'color': "red",
            'size': 10
        },
        'bowler': {
            'color': "black",
            'size': 35
        }
    }
}

PWD = os.path.dirname(os.path.realpath(__file__))


def test_load():

    eq_(load(open(os.path.join(PWD, 'config1.yaml')),
             open(os.path.join(PWD, 'config2.yaml'))),
        EXPECTED)

    eq_(loads(open(os.path.join(PWD, 'config1.yaml')).read(),
              open(os.path.join(PWD, 'config2.yaml')).read()),
        EXPECTED)


@raises(errors.ConfigError)
def test_load():
    load(*[])



import io

import yaml

from . import errors
from .merge import merge
from .propagate_defaults import propagate_defaults


def load(*files):
    """
    Loads configuration from one or more files by merging right to left.

    :Parameters:
        *files : `file-like`
            A YAML file to read.

    :Returns:
        `dict` : the configuration document
    """
    if len(files) == 0:
        raise errors.ConfigError("No config files provided.")
    doc = merge(*(yaml.safe_load(f) for f in files))
    return propagate_defaults(doc)


def loads(*strings):
    """
    Loads configuration from one or more files by merging right to left.

    :Parameters:
        *strings : `str`
            YAML strings to read.

    :Returns:
        `dict` : the configuration document
    """
    return load(*(io.StringIO(s) for s in strings))



from collections import namedtuple

Attribute = namedtuple("Attribute", ["attribute"])

super_sub_attribute = Attribute("hello")
sub_attribute = Attribute(super_sub_attribute)
attribute = Attribute(sub_attribute)



__name__ = "yamlconf"
__version__ = "0.2.5"
__author__ = "Aaron Halfaker"
__author_email__ = "aaron.halfaker@gmail.com"
__description__ = "Yaml-based configuration file reader."
__license__ = "MIT"
__url__ = "https://github.com/halfak/yamlconf"

all = [__name__, __version__, __author__, __author_email__, __description__,
       __license__, __url__]



import copy

from .merge import dict_merge_pair


def propagate_defaults(config_doc):
    """
    Propagate default values to sections of the doc.
    """
    for group_name, group_doc in config_doc.items():
        if isinstance(group_doc, dict):
            defaults = group_doc.get('defaults', {})

            for item_name, item_doc in group_doc.items():
                if item_name == 'defaults':
                    continue
                if isinstance(item_doc, dict):

                    group_doc[item_name] = \
                        dict_merge_pair(copy.deepcopy(defaults), item_doc)

    return config_doc
